Model_Lines = {}

Model_Lines.gameOver = false;

Model_Lines.NN       = 8;
Model_Lines.COLORS   = 6;
Model_Lines.MIN_LINE = 4;

Model_Lines.matrix = null;
Model_Lines.curI   = null;
Model_Lines.curJ   = null;
Model_Lines.count  = null;

Model_Lines.nextColors = null;

Model_Lines.points = null;

Model_Lines.handlerSelect       = null;
Model_Lines.handlerMove         = null;
Model_Lines.handlerNewBall      = null;
Model_Lines.handlerDelLine      = null;
Model_Lines.handlerGenNextColors = null;
Model_Lines.handlerGameOver     = null;

/**
 * Установка обработчиков события модели
 * @param handlers объект, имеющий поля handlerSelect, handlerMove,
 * handlerNewBall, handlerDelLine и handlerGenNextColors
 */
Model_Lines.setHandlers = function(handlers)
{
	Model_Lines.handlerSelect  = handlers.handlerSelect;
	Model_Lines.handlerMove    = handlers.handlerMove;
	Model_Lines.handlerNewBall = handlers.handlerNewBall;
	Model_Lines.handlerDelLine = handlers.handlerDelLine;
	Model_Lines.handlerGenNextColors = handlers.handlerGenNextColors;
	Model_Lines.handlerGameOver = handlers.handlerGameOver;
}

Model_Lines.selected = function()
{
	return (Model_Lines.curI != null);
}

Model_Lines.clear = function()
{
	Model_Lines.handlerSelect  = null;
	Model_Lines.handlerMove    = null;
	Model_Lines.handlerNewBall = null;
	Model_Lines.handlerDelLine = null;
	Model_Lines.handlerGenNextColors = null;
	Model_Lines.handlerGameOver     = null;
	
	Model_Lines.matrix = null;
	Model_Lines.nextColors = null;
}

Model_Lines.startGame = function()
{
	Model_Lines.gameOver = false;
	
	Model_Lines.points = 0;
	Model_Lines.curI   = null;
	Model_Lines.curJ   = null;
	Model_Lines.nextColors = new Array(3);
	
	Model_Lines.matrixInit();
	Model_Lines.genNextColors();
	
	Model_Lines.addNewBalls();
}


Model_Lines.select = function(i, j)
{
	if (! Model_Lines.isFree(i, j)) {
		
		if ((Model_Lines.curI == i) && (Model_Lines.curJ == j)) return;
		
		Model_Lines.curI = i;
		Model_Lines.curJ = j;
		
		Model_Lines.handlerSelect.call(null, i, j);
	}
	
}

Model_Lines.move = function(i, j)
{
	if (Model_Lines.curI != null) {
		
		if (Model_Lines.isFree(i, j)) {
			
			var ci = Model_Lines.curI;
			var cj = Model_Lines.curJ;
			
			if (Model_Lines.checkPath(ci, cj, i, j)) {

				var c = Model_Lines.getColor();
				Model_Lines.setColor(0);
				Model_Lines.curI = null;
				Model_Lines.curJ = null;
				Model_Lines.setColor(c, i, j);
			
				Model_Lines.handlerMove.call(null, ci, cj, i, j);
			}
		}
		else
			Model_Lines.select(i, j);
	}
		
}

Model_Lines.isFree = function(i, j)
{
	return Model_Lines.matrix[ i ][ j ] == 0;
}

Model_Lines.getColor = function (i, j)
{
	if ( (i==undefined) || (j==undefined) ) {
		
		i = Model_Lines.curI;
		j = Model_Lines.curJ;
	}
	
	return Model_Lines.matrix[i][j];
}

Model_Lines.setColor = function(color, i, j)
{
	if ( (i==undefined) || (j==undefined) ) {
		
		i = Model_Lines.curI;
		j = Model_Lines.curJ;
	}
	
	Model_Lines.matrix[i][j] = color;
}




Model_Lines.addNewBalls = function()
{
	
		
	var p;
	var line;
	for (var i=0; i<3; i++){
		
		p = Model_Lines.getFreePoint();
		if (p == null) {
			Model_Lines.gameOver = true;
			Model_Lines.handlerGameOver.call();
			return;
		}
		
		Model_Lines.count++;
		Model_Lines.setColor(Model_Lines.nextColors[i], p[0], p[1]);
		Model_Lines.handlerNewBall.call(
			null, Model_Lines.nextColors[i], p[0], p[1]);
			
		line = Model_Lines.checkLine(p[0], p[1])
		if (line)
			Model_Lines.delLine(line);
	}
		
	Model_Lines.genNextColors();
	
}

Model_Lines.matrixInit = function()
{
	var i, j;
	if (! Model_Lines.matrix) {
		
		Model_Lines.matrix = new Array(8);	
		for (i = 0; i < Model_Lines.NN; i++) {
			Model_Lines.matrix[i] = new Array()
	
			for (j=0; j < Model_Lines.NN; j++)
				Model_Lines.matrix[i][j] = 0;
		}
	}
	else {
		
		for (i=0; i < Model_Lines.NN; i++)
			for (j=0; j < Model_Lines.NN; j++)
				Model_Lines.matrix[i][j]=0;
	}
	
	Model_Lines.count = 0;
}

Model_Lines.getRandomColor = function()
{
	var c = Math.floor( Math.random( ) * (Model_Lines.COLORS) ) + 1;
	return c;
}

Model_Lines.getFreePoint = function()
{
	
	if (Model_Lines.count >= Model_Lines.NN * Model_Lines.NN-1)
		return null;
	
	var ri = Math.floor( Math.random( ) * (Model_Lines.NN) );
	var rj = Math.floor( Math.random( ) * (Model_Lines.NN) );
	
	
	if (Model_Lines.isFree(ri, rj) ) {
		return [ri, rj];
	}
	
	else {
		
		var i,j;
		
		for (i=ri; i < Model_Lines.NN; i++) {
			for (j=rj; j < Model_Lines.NN; j++){
				if ( Model_Lines.isFree(i, j) ) {
					return [i, j];
				}
			}
			rj=0;
		}
					
			
		for (i=ri; i >= 0; i--) {
			for (j=rj; j>=0; j--) {
				
				if ( Model_Lines.isFree(i, j) ) {
					return [i, j];
				}
			}
			rj=Model_Lines.NN-1;
		}
					
			
		return null;
	}
}

Model_Lines.genNextColors = function()
{
	for (var i=0; i<3; i++)
		Model_Lines.nextColors[i] = Model_Lines.getRandomColor();
	
	Model_Lines.handlerGenNextColors.call(null, Model_Lines.nextColors);
}

Model_Lines.checkPath = function(i1, j1, i2, j2)
{
	var matr = new Array( Model_Lines.NN );
	for (var i = 0; i < Model_Lines.NN; i++) {
		
		matr[i] = new Array(8);
		for (var j = 0; j < Model_Lines.NN; j++)
			matr[i][j] = Model_Lines.matrix[i][j] != 0;
	}
	
	return Model_Lines.checkPathStep(i1, j1, i2, j2, matr);
}

Model_Lines.checkPathStep = function(i1, j1, i2, j2, matr)
{
	

	
	if ( (i1 == i2) && (j1 == j2) )
		return true;
	
	matr[i1][j1] = true;
	
	var result = false;
	
	//вниз
	if ((i1+1 < Model_Lines.NN) && (! matr[i1+1][j1])) {

		result = Model_Lines.checkPathStep(i1+1, j1, i2, j2, matr);
	}
	
	//вверх
	if (! result) {
		if ((i1 -1 >= 0) && (! matr[i1-1][j1])) {

			result = Model_Lines.checkPathStep(i1-1, j1, i2, j2, matr);
		}
	}
	
	//вправо
	if (! result) {
		
		if ((j1 + 1 < Model_Lines.NN) && (! matr[i1][j1+1])) {
			
	
			result = Model_Lines.checkPathStep(i1, j1+1, i2, j2, matr);
		}
	}
	
	//влево
	if (! result) {
		
		if ( (j1 - 1 >= 0) && (! matr[i1][j1-1]) ) {
			
		
			result = Model_Lines.checkPathStep(i1, j1-1, i2, j2, matr);
		}
	}
	
	return result;
}


Model_Lines.checkLine = function(ii, jj)
{
	var i, j, k, result;
	var c = Model_Lines.getColor(ii, jj);
	
	//горизонталь 
	k=1;
	result = [ [ii, jj] ];
	//влево
	j = jj-1;
	while (true) {
		
		if (j<0) break;
		if ( c != Model_Lines.getColor(ii, j) ) break;
		
		result.push( [ii, j] );
		k++;
		j--;
	}
	//вправо
	j = jj+1;
	while (true) {
		
		if (j == Model_Lines.NN) break;
		if (c != Model_Lines.getColor(ii, j)) break;
		
		result.push( [ii, j] );
		k++;
		j++;
	}
	
	//горизонталь закончили, смотрим результат
	if ( k >= Model_Lines.MIN_LINE) {
		return result;
	}
	
	//по горизонтали ничего не оказалось
	//вертикаль
	k = 1;
	if (result.length > 1)
		result = [ [ii, jj] ];
	//вверх
	i = ii-1;
	while (true) {
		
		if (i < 0)  break;
		if (c != Model_Lines.getColor(i, jj)) break;
		
		result.push( [i, jj] );
		k++;
		i--;
	}
	//вниз
	i = ii+1;
	while (true) {
		
		if (i == Model_Lines.NN) break;
		if (c != Model_Lines.getColor(i, jj)) break;
		
		result.push( [i, jj] );
		k++;
		i++;
	}
	
	//вертикаль закончили, смотрим результат
	if ( k >= Model_Lines.MIN_LINE )
		return result;
	
	//на вертикали ничего не оказалось.
	//"главная диагональ"
	k=1;
	if (result.length > 1)
		result = [ [ii, jj] ];
	//влево-вверх
	i = ii-1;
	j = jj-1;
	while (true) {
		
		if (i < 0) break;
		if (j < 0) break;
		if (c != Model_Lines.getColor(i, j)) break;
		
		result.push( [i, j] );
		k++;
		i--;j--;
	}
	//вправо-вниз
	i = ii+1;
	j = jj+1;
	while (true) {
		
		if (i == Model_Lines.NN) break;
		if (j == Model_Lines.NN) break;
		if (c != Model_Lines.getColor(i, j)) break;
		
		result.push( [i, j] );
		k++;
		i++;j++;
	}
	
	//"главную" диагональ закончили, смотрим результат
	if ( k >= Model_Lines.MIN_LINE )
		return result;
	
	//на главной диагонали ничего не оказалось
	//"Побочная" диагональ
	k = 1;
	if (result.length > 1)
		result = [ [ii, jj] ];
	//влево-вниз
	i = ii+1;
	j = jj-1;
	while (true){
		
		if (i == Model_Lines.NN) break;
		if (j < 0) break;
		if (c != Model_Lines.getColor(i, j)) break;
		
		result.push( [i, j] );
		k++;
		i++;j--;
	}
	//вправо-вверх
	i = ii-1;
	j = jj+1;
	while (true) {
		
		if (i < 0) break;
		if (j == Model_Lines.NN) break;
		if (c != Model_Lines.getColor(i, j)) break;
		
		result.push( [i, j] );
		k++;
		i--;j++;
	}
	
	//Закончили побочную диагональ, смотрим результат
	if (k >= Model_Lines.MIN_LINE)
		return result;
	
	return null;
}

Model_Lines.delLine = function(line)
{
	for (var i = 0; i < line.length; i++) {
			
		Model_Lines.count--;	
		Model_Lines.setColor(0, line[i][0], line[i][1]);
		Model_Lines.points += 30;
	}
		
	Model_Lines.handlerDelLine.call(null, line);	
}

Model_Lines.handleCourse = function(ii, jj)
{
	var line = Model_Lines.checkLine(ii, jj);

	if (line) {
		Model_Lines.delLine(line);
	}
	else {
		Model_Lines.addNewBalls();
	}
}